using System;
using System.Collections.Generic;
using System.Text;



namespace ComCommonLib {

	/// <summary>
	/// A readable stream.
	/// </summary>
	/// <typeparam name="T">Element type.</typeparam>
	public interface IRead<T> : IOpenClose {

		#region Read -----------------------------------------------------------



		/// <summary>
		/// True if read is buffered
		/// </summary>
		bool ReadBuffered { get; }



		/// <summary>
		/// Buffer size
		/// </summary>
		/// <remarks>
		/// If 0, read is not buffered
		/// </remarks>
		int ReadBufferSize { get; set; }



		/// <summary>
		/// Read timeout
		/// </summary>
		int ReadTimeout { get; set; }



		/// <summary>
		/// Ts in read queue
		/// </summary>
		/// <returns>Number of bytes in read queue</returns>
		int ReadQueue();



		/// <summary>
		/// Cancel pending read and purge read buffer 
		/// </summary>
		void ReadPurge();



		/// <summary>
		/// Read data
		/// </summary>
		/// <param name="buffer">buffer to store the read Ts</param>
		/// <param name="count">Number of Ts to read</param>
		/// <returns>Number of Ts actually read</returns>
		/// <remarks>
		/// Return if all data has been read, timeout elapsed or port is closed
		/// </remarks>
		int Read(T[] buffer, int count);



		/// <summary>
		/// Read T
		/// </summary>
		/// <param name="buffer">buffer to store the read T</param>
		/// <returns>True on success</returns>
		/// <remarks>
		/// Return if data has been read, timeout elapsed or port is closed
		/// </remarks>
		bool Get(out T buffer);



		#endregion

	}
	
	/// <summary>
	/// A readable stream.
	/// </summary>
	public interface IRead : IRead<byte> {
	}

}
