using System;
using System.Collections.Generic;
using System.Text;



namespace ComCommonLib {

	/// <summary>
	/// A writable stream.
	/// </summary>
	/// <typeparam name="T">Element type.</typeparam>
	public interface IWrite<T> : IOpenClose {

		#region Write ----------------------------------------------------------



		/// <summary>
		/// True if write is buffered
		/// </summary>
		bool WriteBuffered { get; }



		/// <summary>
		/// Buffer size
		/// </summary>
		/// <remarks>
		/// If 0, write is not buffered
		/// </remarks>
		int WriteBufferSize { get; set; }



		/// <summary>
		/// Write timeout
		/// </summary>
		int WriteTimeout { get; set; }



		/// <summary>
		/// Ts in write queue
		/// </summary>
		/// <returns>Number of Ts in write queue</returns>
		int WriteQueue();



		/// <summary>
		/// Cancel pending write and purge write buffer 
		/// </summary>
		void WritePurge();



		/// <summary>
		/// Write data
		/// </summary>
		/// <param name="buffer">Ts to write</param>
		/// <param name="count">Length of buffer</param>
		/// <returns>Number of Ts written</returns>
		/// <remarks>
		/// Return if all data has been written, timeout elapsed or port is closed
		/// </remarks>
		int Write(T[] buffer, int count);



		/// <summary>
		/// Write T
		/// </summary>
		/// <param name="buffer">T to write</param>
		/// <returns>True on success</returns>
		/// <remarks>
		/// Return if all data has been written, timeout elapsed or port is closed
		/// </remarks>
		bool Put(ref T buffer);



		#endregion

	}

	/// <summary>
	/// A writable stream.
	/// </summary>
	public interface IWrite : IWrite<byte> {
	}

}
